@@ -11,6 +11,9 @@ from .core import checks
 
 __all__ = [
     "select",
+    "select_mask",
+    "select_indices",
+    "select_labels",
     "expand",
     "overlap",
     "cluster",
@@ -27,6 +30,93 @@ __all__ = [
 ]
 
 
+def select_mask(df, region, cols=None):
+    """
+    Return boolean mask for all genomic intervals that overlap a query range.
+
+    Parameters
+    ----------
+    df : pandas.DataFrame
+
+    region : str or tuple
+        The genomic region to select from the dataframe in UCSC-style genomic
+        region string, or triple (chrom, start, end).
+
+    cols : (str, str, str) or None
+        The names of columns containing the chromosome, start and end of the
+        genomic intervals. The default values are 'chrom', 'start', 'end'.
+
+    Returns
+    -------
+    Boolean array of shape (len(df),)
+    """
+    ck, sk, ek = _get_default_colnames() if cols is None else cols
+    _verify_columns(df, [ck, sk, ek])
+
+    chrom, start, end = parse_region(region)
+
+    if chrom is None:
+        raise ValueError("no chromosome detected, check region input")
+
+    if start is None:
+        mask = df[ck] == chrom
+    else:
+        if end is None:
+            end = np.inf
+        mask = (df[ck] == chrom) & (
+            ((df[sk] < end) & (df[ek] > start)) | 
+            ((df[sk] == df[ek]) & (df[sk] == start))  # include points at query start
+        )
+    return mask.to_numpy()
+
+
+def select_indices(df, region, cols=None):
+    """
+    Return integer indices of all genomic intervals that overlap a query range.
+
+    Parameters
+    ----------
+    df : pandas.DataFrame
+
+    region : str or tuple
+        The genomic region to select from the dataframe in UCSC-style genomic
+        region string, or triple (chrom, start, end).
+
+    cols : (str, str, str) or None
+        The names of columns containing the chromosome, start and end of the
+        genomic intervals. The default values are 'chrom', 'start', 'end'.
+
+    Returns
+    -------
+    1D array of int
+    """
+    return np.nonzero(select_mask(df, region, cols))[0]
+
+
+def select_labels(df, region, cols=None):
+    """
+    Return pandas Index labels of all genomic intervals that overlap a query
+    range.
+
+    Parameters
+    ----------
+    df : pandas.DataFrame
+
+    region : str or tuple
+        The genomic region to select from the dataframe in UCSC-style genomic
+        region string, or triple (chrom, start, end).
+
+    cols : (str, str, str) or None
+        The names of columns containing the chromosome, start and end of the
+        genomic intervals. The default values are 'chrom', 'start', 'end'.
+
+    Returns
+    -------
+    pandas.Index
+    """
+    return df.index[select_mask(df, region, cols)]
+
+
 def select(df, region, cols=None):
     """
     Return all genomic intervals in a dataframe that overlap a genomic region.
@@ -36,10 +126,8 @@ def select(df, region, cols=None):
     df : pandas.DataFrame
 
     region : str or tuple
-        The genomic region to select from the dataframe.
-        UCSC-style genomic region string, or Triple (chrom, start, end),
-        where ``start`` or ``end`` may be ``None``. See :func:`.core.stringops.parse_region()`
-        for more information on region formatting.
+        The genomic region to select from the dataframe in UCSC-style genomic
+        region string, or triple (chrom, start, end).
 
     cols : (str, str, str) or None
         The names of columns containing the chromosome, start and end of the
@@ -49,19 +137,18 @@ def select(df, region, cols=None):
     -------
     df : pandas.DataFrame
 
+    Notes
+    -----
+    See :func:`.core.stringops.parse_region()` for more information on region
+    formatting.
+
+    See also
+    --------
+    :func:`select_mask`
+    :func:`select_indices`
+    :func:`select_labels`
     """
-
-    ck, sk, ek = _get_default_colnames() if cols is None else cols
-    checks.is_bedframe(df, raise_errors=True, cols=[ck, sk, ek])
-
-    chrom, start, end = parse_region(region)
-    if chrom is None:
-        raise ValueError("no chromosome detected, check region input")
-    if (start is not None) and (end is not None):
-        inds = (df[ck] == chrom) & (df[sk] < end) & (df[ek] > start)
-    else:
-        inds = df[ck] == chrom
-    return df[inds]
+    return df.loc[select_mask(df, region, cols)]
 
 
 def expand(df, pad=None, scale=None, side="both", cols=None):
@@ -816,6 +903,7 @@ def _closest_intidxs(
     ignore_overlaps=False,
     ignore_upstream=False,
     ignore_downstream=False,
+    direction_col=None,
     tie_breaking_col=None,
     cols1=None,
     cols2=None,
@@ -897,6 +985,15 @@ def _closest_intidxs(
                 "f(DataFrame) -> Series"
             )
 
+        # Verify and construct the direction_arr (convert from pandas string column to bool array)
+        # TODO: should we add checks that it's valid "strand"?
+        direction_arr = None
+        if direction_col is None:
+            direction_arr = np.ones(len(df1_group), dtype=np.bool_)
+        else:
+            direction_arr = (df1_group[direction_col].values != "-") # both "+" and "." keep orientation by genomic coordinate
+
+        # Calculate closest intervals with arrops:
         closest_idxs_group = arrops.closest_intervals(
             df1_group[sk1].values,
             df1_group[ek1].values,
@@ -907,6 +1004,7 @@ def _closest_intidxs(
             ignore_overlaps=ignore_overlaps,
             ignore_upstream=ignore_upstream,
             ignore_downstream=ignore_downstream,
+            direction=direction_arr
         )
 
         # Convert local per-chromosome indices into the
@@ -934,6 +1032,7 @@ def closest(
     ignore_overlaps=False,
     ignore_upstream=False,
     ignore_downstream=False,
+    direction_col=None,
     tie_breaking_col=None,
     return_input=True,
     return_index=False,
@@ -946,6 +1045,9 @@ def closest(
     """
     For every interval in dataframe `df1` find k closest genomic intervals in dataframe `df2`.
 
+    Currently, we are not taking the feature strands into account for filtering.
+    However, the strand can be used for definition of upstream/downstream of the feature (direction).
+
     Note that, unless specified otherwise, overlapping intervals are considered as closest.
     When multiple intervals are located at the same distance, the ones with the lowest index
     in `df2` are returned.
@@ -957,20 +1059,22 @@ def closest(
         If `df2` is None, find closest non-identical intervals within the same set.
 
     k : int
-        The number of closest intervals to report.
+        The number of the closest intervals to report.
 
     ignore_overlaps : bool
-        If True, return the closest non-overlapping interval.
+        If True, ignore overlapping intervals and return the closest non-overlapping interval.
 
     ignore_upstream : bool
-        If True, ignore intervals in `df2` that are upstream (relative to the
-        reference strand) of intervals in `df1`. Currently, we are not taking
-        the feature strands into account.
+        If True, ignore intervals in `df2` that are upstream of intervals in `df1`,
+        relative to the reference strand or the strand specified by direction_col.
 
     ignore_downstream : bool
-        If True, ignore intervals in `df2` that are downstream (relative to the
-        reference strand) of intervals in `df1`. Currently, we are not taking
-        the feature strands into account.
+        If True, ignore intervals in `df2` that are downstream of intervals in `df1`,
+        relative to the reference strand or the strand specified by direction_col.
+
+    direction_col : str
+        Name of direction column that will set upstream/downstream orientation for each feature.
+        The column should contain bioframe-compliant strand values ("+", "-", ".").
 
     tie_breaking_col : str
         A column in `df2` to use for breaking ties when multiple intervals
@@ -1005,6 +1109,17 @@ def closest(
     df_closest : pandas.DataFrame
         If no intervals found, returns none.
 
+    Notes
+    -----
+    By default, direction is defined by the reference genome: everything with
+    smaller coordinate is considered upstream, everything with larger coordinate
+    is considered downstream.
+
+    If ``direction_col`` is provided, upstream/downstream are relative to the
+    direction column in ``df1``, i.e. features marked "+" and "." strand will
+    define upstream and downstream as above, while features marked "-" have
+    upstream and downstream reversed: smaller coordinates are downstream and
+    larger coordinates are upstream.
     """
 
     if k < 1:
@@ -1031,6 +1146,7 @@ def closest(
         ignore_overlaps=ignore_overlaps,
         ignore_upstream=ignore_upstream,
         ignore_downstream=ignore_downstream,
+        direction_col=direction_col,
         tie_breaking_col=tie_breaking_col,
         cols1=cols1,
         cols2=cols2,
