@@ -64,74 +64,6 @@ def mock_bioframe(num_entries=100):
 
 
 ############# tests #####################
-def test_select():
-    df1 = pd.DataFrame(
-        [["chrX", 3, 8], ["chr1", 4, 5], ["chrX", 1, 5]],
-        columns=["chrom", "start", "end"],
-    )
-
-    region1 = "chr1:4-10"
-    df_result = pd.DataFrame([["chr1", 4, 5]], columns=["chrom", "start", "end"])
-    pd.testing.assert_frame_equal(
-        df_result, bioframe.select(df1, region1).reset_index(drop=True)
-    )
-
-    region1 = "chrX"
-    df_result = pd.DataFrame(
-        [["chrX", 3, 8], ["chrX", 1, 5]], columns=["chrom", "start", "end"]
-    )
-    pd.testing.assert_frame_equal(
-        df_result, bioframe.select(df1, region1).reset_index(drop=True)
-    )
-
-    region1 = "chrX:4-6"
-    df_result = pd.DataFrame(
-        [["chrX", 3, 8], ["chrX", 1, 5]], columns=["chrom", "start", "end"]
-    )
-    pd.testing.assert_frame_equal(
-        df_result, bioframe.select(df1, region1).reset_index(drop=True)
-    )
-
-    ### select with non-standard column names
-    region1 = "chrX:4-6"
-    new_names = ["chr", "chrstart", "chrend"]
-    df1 = pd.DataFrame(
-        [["chrX", 3, 8], ["chr1", 4, 5], ["chrX", 1, 5]],
-        columns=new_names,
-    )
-    df_result = pd.DataFrame(
-        [["chrX", 3, 8], ["chrX", 1, 5]],
-        columns=new_names,
-    )
-    pd.testing.assert_frame_equal(
-        df_result, bioframe.select(df1, region1, cols=new_names).reset_index(drop=True)
-    )
-    region1 = "chrX"
-    pd.testing.assert_frame_equal(
-        df_result, bioframe.select(df1, region1, cols=new_names).reset_index(drop=True)
-    )
-
-    ### select from a DataFrame with NaNs
-    colnames = ["chrom", "start", "end", "view_region"]
-    df = pd.DataFrame(
-        [
-            ["chr1", -6, 12, "chr1p"],
-            [pd.NA, pd.NA, pd.NA, "chr1q"],
-            ["chrX", 1, 8, "chrX_0"],
-        ],
-        columns=colnames,
-    ).astype({"start": pd.Int64Dtype(), "end": pd.Int64Dtype()})
-    df_result = pd.DataFrame(
-        [["chr1", -6, 12, "chr1p"]],
-        columns=colnames,
-    ).astype({"start": pd.Int64Dtype(), "end": pd.Int64Dtype()})
-
-    region1 = "chr1:0-1"
-    pd.testing.assert_frame_equal(
-        df_result, bioframe.select(df, region1).reset_index(drop=True)
-    )
-
-
 def test_trim():
 
     ### trim with view_df
@@ -931,19 +863,6 @@ def test_closest():
         df, bioframe.closest(df1, df2, suffixes=("_1", "_2"), k=2)
     )
 
-    ### closest(df2,df1) ###
-    d = """chrom_1  start_1 end_1   chrom_2 start_2 end_2   distance
-            0   chr1    4   8   chr1    1   5   0
-            1   chr1    10  11  chr1    1   5   5 """
-    df = pd.read_csv(StringIO(d), sep=r"\s+").astype(
-        {
-            "start_2": pd.Int64Dtype(),
-            "end_2": pd.Int64Dtype(),
-            "distance": pd.Int64Dtype(),
-        }
-    )
-    pd.testing.assert_frame_equal(df, bioframe.closest(df2, df1, suffixes=("_1", "_2")))
-
     ### change first interval to new chrom ###
     df2.iloc[0, 0] = "chrA"
     d = """chrom start   end     chrom_ start_ end_  distance
@@ -1039,6 +958,74 @@ def test_closest():
         df1.iloc[0, 0] = "chr10"
         bioframe.closest(df1, df2)
 
+    ### closest with direction ###
+
+    df1 = pd.DataFrame(
+        [
+            ["chr1", 3, 5, "+"],
+            ["chr1", 3, 5, "-"],
+        ],
+        columns=["chrom", "start", "end", "strand"],
+    )
+
+    df2 = pd.DataFrame(
+        [["chr1", 1, 2], ["chr1", 2, 8], ["chr1", 10, 11]], columns=["chrom", "start", "end"]
+    )
+
+    ### closest(df1, df2, k=1, direction_col="strand") ###
+    d = """chrom  start  end strand chrom_  start_  end_  distance
+        0    chr1        3      5    + chr1        2      8         0
+        1    chr1        3      5    - chr1        2      8         0
+        """
+    df = pd.read_csv(StringIO(d), sep=r"\s+").astype(
+        {
+            "start_": pd.Int64Dtype(),
+            "end_": pd.Int64Dtype(),
+            "distance": pd.Int64Dtype(),
+        }
+    )
+    pd.testing.assert_frame_equal(df, bioframe.closest(df1, df2, k=1, direction_col="strand"))
+
+    ### closest(df1, df2, k=1, ignore_upstream=False, ignore_downstream=True, ignore_overlaps=True, direction_col="strand") ###
+    d = """chrom  start  end strand chrom_  start_  end_  distance
+        0    chr1        3      5    + chr1        1      2         1
+        1    chr1        3      5    - chr1        10      11         5
+        """
+    df = pd.read_csv(StringIO(d), sep=r"\s+").astype(
+        {
+            "start_": pd.Int64Dtype(),
+            "end_": pd.Int64Dtype(),
+            "distance": pd.Int64Dtype(),
+        }
+    )
+    pd.testing.assert_frame_equal(df,
+        bioframe.closest(df1, df2,
+                k=1,
+                ignore_upstream=False,
+                ignore_downstream=True,
+                ignore_overlaps=True,
+                direction_col="strand"))
+
+    ### closest(df1, df2, k=1, ignore_upstream=True, ignore_downstream=False, ignore_overlaps=True, direction_col="strand") ###
+    d = """chrom  start  end strand chrom_  start_  end_  distance
+        0    chr1        3      5    + chr1        10      11         5
+        1    chr1        3      5    - chr1        1      2         1
+        """
+    df = pd.read_csv(StringIO(d), sep=r"\s+").astype(
+        {
+            "start_": pd.Int64Dtype(),
+            "end_": pd.Int64Dtype(),
+            "distance": pd.Int64Dtype(),
+        }
+    )
+    pd.testing.assert_frame_equal(df,
+        bioframe.closest(df1, df2,
+                k=1,
+                ignore_upstream=True,
+                ignore_downstream=False,
+                ignore_overlaps=True,
+                direction_col="strand"))
+
 
 def test_coverage():
 
